﻿using System.Windows;
using System.Windows.Media.Animation;
using System.Windows.Threading;

//====================================================
// Описание работы классов и методов исходника на:
// https://www.interestprograms.ru
// Исходные коды программ и игр
//====================================================

namespace WpfAppRocket
{
	/// <summary>
	/// Interaction logic for MainWindow.xaml
	/// </summary>
	public partial class MainWindow : Window
	{
		// Анимационный объект,
		// глобальный для управления его вращением
		// для расположения вдоль оси движения.
		private readonly AnimRocket animObject;
		// Продолжительность аниации объекта
		private double _duration = 2;
		private readonly Random random = new();


		public MainWindow()
		{
			InitializeComponent();

			//AllowsTransparency = true;
			//WindowStyle = WindowStyle.None;
			//Background = Brushes.Transparent;

			animObject = new AnimRocket(mainGrid);

			// Определение максимальных размеров анимационного объекта.
			Size _maxImgSize = animObject.maxImgSize;

			// Основное окно будет квадратным,
			// поскольку объект будет вращаться.
			Width = Height = Math.Max(_maxImgSize.Width, _maxImgSize.Height);

			// Запуск процесса анимации
			AnimationWindow();
		}


		/// <summary>
		/// Анимация перемещения окна
		/// </summary>
		private void AnimationWindow()
		{
			// Габариты перемещения основного окна
			double minX = 0;
			double maxX = SystemParameters.PrimaryScreenWidth;
			double minY = 0;
			double maxY = SystemParameters.PrimaryScreenHeight;

			// Получение координат перемещения объекта 
			(double fromX, double toX, double fromY, double toY) = ComputeCoordinates(minX, maxX, minY, maxY);

			// Вычисляем угол поворота объекта относительно горизонтали.
			Vector vectorBase = new(30, 0);
			Vector vectorDirection = new(toX - fromX, toY - fromY);
			double angle = Vector.AngleBetween(vectorBase, vectorDirection);
			animObject.Rotate(angle);


			// Координата Х расположения окна на экране.
			DoubleAnimation posLeft = new();
			// Координата Y расположения окна на экране.
			DoubleAnimation posTop = new();

			// Характеристики поведения координат идентичны.
			posLeft.Duration = posTop.Duration = new Duration(TimeSpan.FromSeconds(_duration));
			posLeft.FillBehavior = posTop.FillBehavior = FillBehavior.Stop;

			// Позиция окна вычисляется относительно его центра.
			posLeft.From = fromX - Width / 2;
			posLeft.To = toX - Width / 2;
			posTop.From = fromY - Height / 2;
			posTop.To = toY - Height / 2;


			// Анимация появления окна
			double opacityShow = 0.2;
			DoubleAnimation opacityAnimationShow = new()
			{
				From = 0.0,
				To = 1.0,
				Duration = new Duration(TimeSpan.FromSeconds(opacityShow)),
				FillBehavior = FillBehavior.Stop
			};


			// Анимация исчезновения окна.
			double opacityHide = 0.2;
			// Защита от отрицательного временного промежутка.
			double beginTimeHide = _duration >= (opacityShow + opacityHide) ? (_duration - (opacityShow + opacityHide)) : 0.0;
			DoubleAnimation opacityAnimationHide = new()
			{
				From = 1.0,
				To = 0.0,
				Duration = new Duration(TimeSpan.FromSeconds(opacityHide)),
				BeginTime = TimeSpan.FromSeconds(beginTimeHide),
				FillBehavior = FillBehavior.Stop
			};

			// Событие окончания перемещения
			posLeft.Completed += (sender, eArgs) =>
			{
				// Запуск новой анимации
				DispatcherTimer timer = new() { Interval = TimeSpan.FromSeconds(2) };
				timer.Tick += (sender, args) =>
				{
					timer.Stop();

					AnimationWindow();
				};

				timer.Start();
			};

			// Событие появления окна
			opacityAnimationShow.Completed += (sender, eArgs) =>
			{
				this.Opacity = 1;

				// Повторная анимация прозрачности запускается точно после окончания первой,
				// иначе управление одним свойством двумя анимациями создает артефакты.
				BeginAnimation(OpacityProperty, opacityAnimationHide);
			};

			// Окончание исчезновения.
			opacityAnimationHide.Completed += (sender, eArgs) =>
			{
				Opacity = 0;
			};



			// Запуск анимаций
			BeginAnimation(OpacityProperty, opacityAnimationShow);
			BeginAnimation(LeftProperty, posLeft);
			BeginAnimation(TopProperty, posTop);

		}


		/// <summary>
		/// Подготовка координат для перемещения анимации объекта
		/// </summary>
		/// <param name="minX"></param>
		/// <param name="maxX"></param>
		/// <param name="minY"></param>
		/// <param name="maxY"></param>
		/// <returns></returns>
		private (double fromX, double toX, double fromY, double toY) ComputeCoordinates(double minX, double maxX, double minY, double maxY)
		{
			//Console.Beep(3000, 100);

			// Подготовка для выбора случайных координат начала и конца перемещения.
			List<int> randPos = new() { 0, 1, 2, 3 };
			randPos = randPos.OrderBy(a => random.Next()).ToList();

			// Получение случайной координаты старта.
			Point start = GetPointMove(minX, maxX, minY, maxY, randPos[0]);
			double fromX = start.X;
			double fromY = start.Y;

			// Получение координаты финиша.
			Point end = GetPointMove(minX, maxX, minY, maxY, randPos[1]);
			double toX = end.X;
			double toY = end.Y;

			// Получение длины вектора перемещения
			Vector vecDirection = new()
			{
				X = toX - fromX,
				Y = toY - fromY,
			};

			// Скорость перемещения будет всегда одинакова,
			// независимо от расстояния перемещения.
			_duration = vecDirection.Length / 1100;


			return (fromX, toX, fromY, toY);
		}


		/// <summary>
		/// Получение одной координаты перемещения 
		/// </summary>
		/// <param name="minX"></param>
		/// <param name="maxX"></param>
		/// <param name="minY"></param>
		/// <param name="maxY"></param>
		/// <param name="variant">выбор варианта перемещения</param>
		/// <returns></returns>
		private Point GetPointMove(double minX, double maxX, double minY, double maxY, int variant)
		{
			// Количество координатных точек на выбранную сторону экрана.
			int num = 10;

			// Список хранения точек
			List<Point> points = new();


			// Варианты получения координатных точек.
			switch (variant)
			{
				case 0:
					// Верхняя сторона экрана.
					for (int i = 0; i <= num; i++)
					{
						Point p = new() { X = maxX / num * i, Y = minY };
						points.Add(p);
					}
					break;

				case 1:
					// Нижняя сторона экрана
					for (int i = 0; i <= num; i++)
					{
						Point p = new() { X = maxX / num * i, Y = maxY };
						points.Add(p);
					}
					break;

				case 2:
					// Левая сторона экрана
					for (int i = 0; i <= num; i++)
					{
						Point p = new() { X = minX, Y = maxY / num * i };
						points.Add(p);
					}
					break;

				case 3:
					// Правая сторона экрана
					for (int i = 0; i <= num; i++)
					{
						Point p = new() { X = maxX, Y = maxY / num * i };
						points.Add(p);
					}
					break;
			}

			// Случайный выбор одной точки для возрата.
			return points[random.Next(0, points.Count)];
		}


	}
}